=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2008-05-08 04:33:38 +0000
+++ bzrlib/revisiontree.py	2008-05-28 07:01:06 +0000
@@ -42,6 +42,7 @@
         self._weave_store = branch.weave_store
         self._inventory = inv
         self._revision_id = revision_id
+        self._rules_searcher = None
 
     def supports_tree_reference(self):
         return True
@@ -203,3 +204,10 @@
             for dir in reversed(dirblock):
                 if dir[2] == _directory:
                     pending.append(dir)
+
+    def _get_rules_searcher(self, default_searcher):
+        """See Tree._get_rules_searcher."""
+        if self._rules_searcher is None:
+            self._rules_searcher = super(RevisionTree,
+                self)._get_rules_searcher(default_searcher)
+        return self._rules_searcher

=== modified file 'bzrlib/rules.py'
--- bzrlib/rules.py	2008-05-28 02:44:55 +0000
+++ bzrlib/rules.py	2008-05-28 07:01:06 +0000
@@ -27,6 +27,10 @@
 from bzrlib.util.configobj import configobj
 
 
+# Name of the file holding rules in a tree
+RULES_TREE_FILENAME = ".bzrrules"
+
+
 class _RulesSearcher(object):
     """An object that provides rule-based preferences."""
 
@@ -48,9 +52,12 @@
     def __init__(self, inifile):
         """Construct a _RulesSearcher based on an ini file.
 
+        The content will be decoded as utf-8.
+
         :param inifile: the name of the file or a sequence of lines.
         """
-        self._cfg = configobj.ConfigObj(inifile)
+        options = {'encoding': 'utf-8'}
+        self._cfg = configobj.ConfigObj(inifile, options=options)
         patterns = self._cfg.keys()
         if patterns:
             self._globster = globbing._OrderedGlobster(patterns)
@@ -97,45 +104,3 @@
 
 # The object providing default rules
 _per_user_searcher = _IniBasedRulesSearcher(rules_filename())
-
-
-# The cache of branch-specific rule searchers
-_branch_searchers = {}
-
-
-def iter_search_rules(branch, path_names, pref_names=None,
-    _default_searcher=_per_user_searcher):
-    """Find the preferences for filenames in a branch.
-
-    :param branch: the branch, or None to only search the per user preferences
-    :param path_names: an iterable of paths to find attributes for.
-      Paths are given relative to the root of the tree.
-    :param pref_names: the list of preferences to lookup - None for all
-    :param _default_searcher: private parameter to assist testing - don't use
-    :return: an iterator of tuple sequences, one per path-name.
-      See _RulesSearcher.get_items for details on the tuple sequence.
-    """
-    # Get the searcher
-    if branch is None:
-        searcher = _default_searcher
-    else:
-        branch_searcher = _branch_searchers.get(branch)
-        if branch_searcher is None:
-            # Create and cache the branch searcher
-            branch.lock_read()
-            try:
-                ini_file = branch.control_files.get('branch.rules')
-            finally:
-                branch.unlock()
-            branch_searcher = _IniBasedRulesSearcher(ini_file)
-            _branch_searchers[branch] = branch_searcher
-        # If branch.rules is missing or empty, skip searching it
-        if branch_searcher._globster is None:
-            searcher = _default_searcher
-        else:
-            searcher = _StackedRulesSearcher(
-                [branch_searcher, _default_searcher])
-
-    # Do the searching
-    for path in path_names:
-        yield searcher.get_items(path, pref_names)

=== modified file 'bzrlib/tests/test_rules.py'
--- bzrlib/tests/test_rules.py	2008-05-28 02:44:55 +0000
+++ bzrlib/tests/test_rules.py	2008-05-28 07:01:06 +0000
@@ -106,54 +106,3 @@
     def test_rules_filename(self):
         self.assertEqual(rules.rules_filename(),
                          self.bzr_home + '/rules')
-
-
-class TestIterSearchRules(tests.TestCaseInTempDir):
-
-    def make_per_user_searcher(self, lines):
-        """Make a _RulesSearcher from a list of strings"""
-        return rules._IniBasedRulesSearcher(lines)
-
-    def make_branch_with_rules(self, text):
-        b = self.make_branch('.')
-        control_files = b.control_files
-        control_files.lock_write()
-        try:
-            control_files.put_utf8('branch.rules', text)
-        finally:
-            control_files.unlock()
-        return b
-
-    def test_iter_search_rules_no_branch(self):
-        per_user = self.make_per_user_searcher([
-            "[./a.txt]", "foo=baz",
-            "[*.txt]", "foo=bar", "a=True"])
-        result = list(rules.iter_search_rules(None, ['a.txt', 'dir/a.txt'],
-            _default_searcher=per_user))
-        self.assertEquals((('foo', 'baz'),), result[0])
-        self.assertEquals((('foo', 'bar'), ('a', 'True')), result[1])
-
-    def test_iter_search_rules_just_branch(self):
-        per_user = self.make_per_user_searcher([])
-        b = self.make_branch_with_rules(
-            "[./a.txt]\n"
-            "foo=baz\n"
-            "[*.txt]\n"
-            "foo=bar\n"
-            "a=True\n")
-        result = list(rules.iter_search_rules(b, ['a.txt', 'dir/a.txt'],
-            _default_searcher=per_user))
-        self.assertEquals((('foo', 'baz'),), result[0])
-        self.assertEquals((('foo', 'bar'), ('a', 'True')), result[1])
-
-    def test_iter_search_rules_branch_and_per_user(self):
-        per_user = self.make_per_user_searcher([
-            "[./a.txt]", "foo=baz",
-            "[*.txt]", "foo=bar", "a=True"])
-        b = self.make_branch_with_rules(
-            "[./a.txt]\n"
-            "foo=qwerty\n")
-        result = list(rules.iter_search_rules(b, ['a.txt', 'dir/a.txt'],
-            _default_searcher=per_user))
-        self.assertEquals((('foo', 'qwerty'),), result[0])
-        self.assertEquals((('foo', 'bar'), ('a', 'True')), result[1])

=== modified file 'bzrlib/tests/tree_implementations/__init__.py'
--- bzrlib/tests/tree_implementations/__init__.py	2008-05-15 14:11:54 +0000
+++ bzrlib/tests/tree_implementations/__init__.py	2008-05-28 07:01:06 +0000
@@ -352,6 +352,7 @@
         'bzrlib.tests.tree_implementations.test_get_root_id',
         'bzrlib.tests.tree_implementations.test_get_symlink_target',
         'bzrlib.tests.tree_implementations.test_inv',
+        'bzrlib.tests.tree_implementations.test_iter_search_rules',
         'bzrlib.tests.tree_implementations.test_list_files',
         'bzrlib.tests.tree_implementations.test_path_content_summary',
         'bzrlib.tests.tree_implementations.test_revision_tree',

=== added file 'bzrlib/tests/tree_implementations/test_iter_search_rules.py'
--- bzrlib/tests/tree_implementations/test_iter_search_rules.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/tree_implementations/test_iter_search_rules.py	2008-05-28 07:01:06 +0000
@@ -0,0 +1,77 @@
+# Copyright (C) 2008 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test that all Tree's implement iter_search_rules."""
+
+from bzrlib import (
+    rules,
+    tests,
+    )
+from bzrlib.tests.tree_implementations import TestCaseWithTree
+
+
+class TestIterSearchRules(TestCaseWithTree):
+
+    def make_per_user_searcher(self, lines):
+        """Make a _RulesSearcher from a list of strings"""
+        return rules._IniBasedRulesSearcher(lines)
+
+    def make_tree_with_rules(self, text):
+        tree = self.make_branch_and_tree('.')
+        if text is not None:
+            text_utf8 = text.encode('utf-8')
+            self.build_tree_contents([(rules.RULES_TREE_FILENAME, text_utf8)])
+            tree.add(rules.RULES_TREE_FILENAME)
+            tree.commit("add rules file")
+        result = self._convert_tree(tree)
+        result.lock_read()
+        self.addCleanup(result.unlock)
+        return result
+
+    def test_iter_search_rules_no_tree(self):
+        per_user = self.make_per_user_searcher([
+            "[./a.txt]", "foo=baz",
+            "[*.txt]", "foo=bar", "a=True"])
+        tree = self.make_tree_with_rules(None)
+        result = list(tree.iter_search_rules(['a.txt', 'dir/a.txt'],
+            _default_searcher=per_user))
+        self.assertEquals((('foo', 'baz'),), result[0])
+        self.assertEquals((('foo', 'bar'), ('a', 'True')), result[1])
+
+    def test_iter_search_rules_just_tree(self):
+        per_user = self.make_per_user_searcher([])
+        tree = self.make_tree_with_rules(
+            "[./a.txt]\n"
+            "foo=baz\n"
+            "[*.txt]\n"
+            "foo=bar\n"
+            "a=True\n")
+        result = list(tree.iter_search_rules(['a.txt', 'dir/a.txt'],
+            _default_searcher=per_user))
+        self.assertEquals((('foo', 'baz'),), result[0])
+        self.assertEquals((('foo', 'bar'), ('a', 'True')), result[1])
+
+    def test_iter_search_rules_tree_and_per_user(self):
+        per_user = self.make_per_user_searcher([
+            "[./a.txt]", "foo=baz",
+            "[*.txt]", "foo=bar", "a=True"])
+        tree = self.make_tree_with_rules(
+            "[./a.txt]\n"
+            "foo=qwerty\n")
+        result = list(tree.iter_search_rules(['a.txt', 'dir/a.txt'],
+            _default_searcher=per_user))
+        self.assertEquals((('foo', 'qwerty'),), result[0])
+        self.assertEquals((('foo', 'bar'), ('a', 'True')), result[1])

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2008-05-15 14:11:54 +0000
+++ bzrlib/tree.py	2008-05-28 07:01:06 +0000
@@ -23,10 +23,11 @@
 
 import bzrlib
 from bzrlib import (
+    conflicts as _mod_conflicts,
     delta,
     osutils,
     revision as _mod_revision,
-    conflicts as _mod_conflicts,
+    rules,
     symbol_versioning,
     )
 from bzrlib.decorators import needs_read_lock
@@ -509,6 +510,33 @@
         """
         raise NotImplementedError(self.walkdirs)
 
+    def iter_search_rules(self, path_names, pref_names=None,
+        _default_searcher=rules._per_user_searcher):
+        """Find the preferences for filenames in a tree.
+
+        :param path_names: an iterable of paths to find attributes for.
+          Paths are given relative to the root of the tree.
+        :param pref_names: the list of preferences to lookup - None for all
+        :param _default_searcher: private parameter to assist testing - don't use
+        :return: an iterator of tuple sequences, one per path-name.
+          See _RulesSearcher.get_items for details on the tuple sequence.
+        """
+        searcher = self._get_rules_searcher(_default_searcher)
+        if searcher is not None:
+            for path in path_names:
+                yield searcher.get_items(path, pref_names)
+
+    @needs_read_lock
+    def _get_rules_searcher(self, default_searcher):
+        """Get the RulesSearcher for this tree given the default one."""
+        searcher = default_searcher
+        file_id = self.path2id(rules.RULES_TREE_FILENAME)
+        if file_id is not None:
+            ini_file = self.get_file(file_id)
+            searcher = rules._StackedRulesSearcher(
+                [rules._IniBasedRulesSearcher(ini_file), default_searcher])
+        return searcher
+
 
 class EmptyTree(Tree):
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2008-05-08 04:33:38 +0000
+++ bzrlib/workingtree.py	2008-05-28 07:01:06 +0000
@@ -245,6 +245,7 @@
             # permitted to do this.
             self._set_inventory(_inventory, dirty=False)
         self._detect_case_handling()
+        self._rules_searcher = None
 
     def _detect_case_handling(self):
         wt_trans = self.bzrdir.get_workingtree_transport(None)
@@ -2435,6 +2436,14 @@
         """
         return
 
+    @needs_read_lock
+    def _get_rules_searcher(self, default_searcher):
+        """See Tree._get_rules_searcher."""
+        if self._rules_searcher is None:
+            self._rules_searcher = super(WorkingTree,
+                self)._get_rules_searcher(default_searcher)
+        return self._rules_searcher
+
 
 class WorkingTree2(WorkingTree):
     """This is the Format 2 working tree.

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2008-05-08 04:33:38 +0000
+++ bzrlib/workingtree_4.py	2008-05-28 07:01:06 +0000
@@ -147,6 +147,7 @@
         #-------------
         self._setup_directory_is_tree_reference()
         self._detect_case_handling()
+        self._rules_searcher = None
 
     @needs_tree_write_lock
     def _add(self, files, ids, kinds):

